/**
 * @file
 * Provides Splide extensions for (local|remote) videos.
 *
 * This extension is using Blazy media player, not the Splide video extension
 * for reasons: already existed years ago, cross-modules, not only Splide,
 * php-based for cacheability, less size, etc. The caveat is lacking of options.
 * However the options respect Splide conventions to easily override/ switch.
 */

(function ($, _ds, _win) {

  'use strict';

  var Media = function (Splide, Components) {
    var root = Splide.root;
    var o = Splide.options;
    var ov = o.video || {};
    var _autoVideo = ov.autoplay || false;
    var _videoTimer;
    var _interrupted = false;
    var _isPlaying = 'is-playing';
    var _isPaused = 'is-paused';
    var icon = '.media__icon';
    var btnClose = icon + '--close';
    var btnPlay = icon + '--play';

    return {
      mount: function () {
        var me = this;

        Splide.on('active.spm', me.play.bind(me));
        Splide.on('moved.spm', me.close.bind(me));
        $.on(root, 'click', btnClose, me.stop.bind(me));
        $.on(root, 'click', btnPlay, me.pause.bind(me));
      },

      /**
       * Turns off any playing (local|remote) videos.
       */
      close: function () {
        var me = this;

        me.stop();
        me.stopLocalVideo();
        _interrupted = false;
      },

      play: function (slide) {
        var me = this;

        if (!_autoVideo || _interrupted) {
          return;
        }

        var media = $.find(slide.slide, '.media');
        if ($.isElm(media)) {
          me.close();

          var btn = $.find(media, btnPlay);
          // var vid = $.find(media, 'video');
          if ($.isElm(btn)) {
            me._play(btn);
          }
          // @fixme, not as expected.
          // else if ($.isElm(vid)) {
          // me._play(vid);
          // }
        }
      },

      _play: function (el) {
        _win.clearTimeout(_videoTimer);
        _videoTimer = _win.setTimeout(function () {

          el[$.equal(el, 'video') ? 'play' : 'click']();
        }, 501);
      },

      /**
       * Pauses the local video.
       */
      stopLocalVideo: function () {
        var vids = $.findAll(root, 'video');
        if (vids.length) {
          $.each(vids, function (vid) {
            vid.pause();
          });
        }
      },

      /**
       * Stops the remote video.
       */
      stop: function () {
        // Clean up any pause marker at slider container.
        $.removeClass(root, _isPaused);

        var cn = $.find(root, '.' + _isPlaying);
        if ($.isElm(cn)) {
          $.removeClass(cn, _isPlaying);
          var btn = $.find(cn, btnClose);

          if ($.isElm(btn)) {
            _interrupted = true;
            btn.click();
          }
        }
      },

      /**
       * Trigger pause on splide instance when media playing a video.
       */
      pause: function () {
        _interrupted = true;
        $.addClass(root, _isPaused);
        if (o.autoplay) {
          Splide.off('autoplay:playing');
        }
      }
    };
  };

  _ds.extend({
    Media: Media
  });

})(dBlazy, dSplide, this);
