<?php

namespace Drupal\splide\Plugin\Field\FieldFormatter;

use Drupal\Component\Utility\Xss;
use Drupal\blazy\BlazyDefault;
use Drupal\blazy\Field\BlazyField;

/**
 * A Trait common for splide vanilla formatters.
 */
trait SplideVanillaWithNavTrait {

  /**
   * {@inheritdoc}
   */
  public function buildElement(array &$build, $entity, $langcode) {
    parent::buildElement($build, $entity, $langcode);

    // Supports Vanilla with thumbnail navigation.
    $settings = $this->formatter->toHashtag($build);
    $blazies  = $settings['blazies'];
    $is_nav   = $blazies->is('nav') ?: $settings['nav'] ?? FALSE;

    if (!empty($settings['vanilla']) && $is_nav) {
      $delta = $blazies->get('delta');
      $element = [
        '#delta'    => $delta,
        '#entity'   => $entity,
        '#settings' => $settings,
        '#item'     => NULL,
      ];
      // Build media item including custom highres video thumbnail.
      // @todo re-check/ refine for Paragraphs, etc.
      $this->blazyOembed->build($element);

      $this->buildElementThumbnail($build, $element, $entity, $delta);
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function buildElementThumbnail(array &$build, array $element, $entity, $delta) {
    // The settings in $element has updated metadata extracted from media.
    $settings  = $this->formatter->toHashtag($element);
    $blazies   = $settings['blazies'];
    $is_nav    = $blazies->is('nav') || !empty($settings['nav']);
    $item      = $this->formatter->toHashtag($element, 'item', NULL);
    $view_mode = $settings['view_mode'] ?? 'default';
    $_caption  = $settings['nav_caption'] ?? NULL;
    $caption   = [];

    // @todo recheck any other places calling this method, and remove this.
    if (!$is_nav) {
      return;
    }

    if ($_caption) {
      if ($item && $text = trim($item->{$_caption} ?? '')) {
        $caption = ['#markup' => Xss::filterAdmin($text)];
      }
      else {
        $caption = BlazyField::view($entity, $_caption, $view_mode);
      }
    }

    $tn = $this->formatter->getThumbnail($settings, $item, $caption);

    // Thumbnail usages: asNavFor pagers, dot, arrows, photobox thumbnails.
    $build[static::$navId]['items'][$delta] = $tn;
  }

  /**
   * If text pagination is configured, pass strings to the JavaScript.
   */
  protected function checkTextPagination(array $entities) {
    if ($pagination_text = $this->getSetting('pagination_text')) {
      $pagination_texts = [];
      foreach ($entities as $entity) {
        if (!isset($entity->{$pagination_text})) {
          continue;
        }
        if ($field = $entity->get($pagination_text)) {
          $value = $field->getString();
          $value = $value ? Xss::filter($value, BlazyDefault::TAGS) : NULL;
          $pagination_texts[] = $value ?: $this->t('Missing navigation label!');
        }
      }

      if ($pagination_texts) {
        $this->setSetting('pagination_texts', $pagination_texts);
      }
    }
  }

}
